@@ -70,6 +70,7 @@ class Feed:
         self.url = url
         self.countUnread = 0
         self.updateTime = "Never"
+        self.updateStamp = 0
         self.uniqueId = uniqueId
         self.etag = None
         self.modified = None
@@ -132,17 +133,20 @@ class Feed:
             tmp=feedparser.parse(self.url, etag = self.etag, modified = self.modified)
         else:
             tmp=feedparser.parse(self.url, etag = self.etag, modified = self.modified, handlers = [proxy])
-        try:
-            self.etag = tmp["etag"]
-        except KeyError:
-            pass
-        try:
-            self.modified = tmp["modified"]
-        except KeyError:
-            pass
         expiry = float(expiryTime) * 3600.
+
         # Check if the parse was succesful (number of entries > 0, else do nothing)
         if len(tmp["entries"])>0:
+           # The etag and modified value should only be updated if the content was not null
+           try:
+               self.etag = tmp["etag"]
+           except KeyError:
+               self.etag = None
+           try:
+               self.modified = tmp["modified"]
+           except KeyError:
+               self.modified = None
+           #if len(tmp["entries"])>0:
            if not isdir(configdir+self.uniqueId+".d"):
                mkdir(configdir+self.uniqueId+".d")
            try:
@@ -251,6 +255,7 @@ class Feed:
            del tmp
            self.countUnread = tmpUnread
            self.updateTime = time.asctime()
+           self.updateStamp = currentTime
            self.saveFeed(configdir)
            from glob import glob
            from os import stat
@@ -313,7 +318,9 @@ class Feed:
             self.readItems[id] = False
     
     def isEntryRead(self, id):
-        return self.readItems[id]
+        # Check if an entry is read; return False if the read
+        # status of an entry is unknown (id not in readItems)
+        return self.readItems.get(id, False)
     
     def getTitle(self, id):
         return self.entries[id]["title"]
@@ -341,6 +348,13 @@ class Feed:
     def getUpdateTime(self):
         return self.updateTime
     
+    def getUpdateStamp(self):
+        try:
+            return self.updateStamp
+        except:
+            self.updateStamp = 0
+            return self.updateStamp
+
     def getEntries(self):
         return self.entries
     
@@ -475,6 +489,7 @@ class ArchivedArticles(Feed):
             #        if currentTime - entry["time"] > 2*expiry:
             #            self.removeEntry(id)
         self.updateTime = time.asctime()
+        self.updateStamp = time.time()
         self.saveFeed(configdir)
         
     def purgeReadArticles(self):
@@ -568,12 +583,14 @@ class Listing:
             feed.updateFeed(self.configdir, expiryTime, proxy, imageCache)
             self.listOfFeeds[key]["unread"] = feed.getNumberOfUnreadItems()
             self.listOfFeeds[key]["updateTime"] = feed.getUpdateTime()
+            self.listOfFeeds[key]["updateStamp"] = feed.getUpdateStamp()
             
     def updateFeed(self, key, expiryTime=24, proxy=None, imageCache=False):
         feed = self.getFeed(key)
         feed.updateFeed(self.configdir, expiryTime, proxy, imageCache)
         self.listOfFeeds[key]["unread"] = feed.getNumberOfUnreadItems()
         self.listOfFeeds[key]["updateTime"] = feed.getUpdateTime()
+        self.listOfFeeds[key]["updateStamp"] = feed.getUpdateStamp()
         
     def editFeed(self, key, title, url):
         self.listOfFeeds[key]["title"] = title
@@ -607,6 +624,12 @@ class Listing:
             self.listOfFeeds[key]["updateTime"] = "Never"
         return self.listOfFeeds[key]["updateTime"]
     
+    def getFeedUpdateStamp(self, key):
+        #print self.listOfFeeds.has_key(key)
+        if not self.listOfFeeds[key].has_key("updateStamp"):
+            self.listOfFeeds[key]["updateStamp"] = 0
+        return self.listOfFeeds[key]["updateStamp"]
+
     def getFeedNumberOfUnreadItems(self, key):
         if not self.listOfFeeds[key].has_key("unread"):
             self.listOfFeeds[key]["unread"] = 0
